<?php


namespace FiloBlu\OrderFlowRectifier\Model\Rectifiers;

use FiloBlu\OrderFlowRectifier\Api\RectifierInterface;

class CompleteKlarnaPaymentsDetailsRectifier implements RectifierInterface
{
    const METHOD_CODE = 'method_code';
    const AUTHORIZED_PAYMENT_METHOD = 'authorized_payment_method';
    const MAPPED_KLARNA_PAYMENT = [
        'klarna_pay_later_direct_bank_transfer' => 'klarna_paynow',
        'klarna_pay_later_slice_it_by_card' => 'klarna_payinthree',
        'klarna_pay_later_invoice' => 'klarna_paylater',
        'klarna_pay_later_pix' => 'klarna_financing',
        'klarna_pay_now_direct_bank_transfer' => 'klarna_paynow',
        'klarna_pay_over_time_fixed_amount' => 'klarna_financing',
        'klarna_pay_over_time_base_account' => 'klarna_financing',
        'klarna_pay_over_time_pay_by_card' => 'klarna_paynow',
        'klarna_pay_over_time_slice_it_by_card' => 'klarna_payinthree'
    ];
    protected $connection;

    public function __construct(
        \Magento\Framework\App\ResourceConnection $connection
    )
    {
        $this->connection = $connection->getConnection();
    }

    public function getInfo()
    {
        return 'Update Klarna payments details with method code name';
    }

    public function getTitle()
    {
        return 'Complete Klarna Payments Details';
    }

    public function executeSingle($order)
    {
        $orderPaymentTable = $this->connection->getTableName('sales_order_payment');
        $orderTableGrid = $this->connection->getTableName('sales_order_grid');
        $payment_additional_information = json_decode($order->additional_information, true);
        $affectedRows = 0;

        /*
         * NOTE: For using this fix in the project must be applied the
         * "klarna_add_authorized_payment_method_on_additional_info_v6_new.patch" or
         * "klarna_add_authorized_payment_method_on_additional_info_v7_new.patch" patch,
         * depending on klarna/module-kp module version.
         */
        if(!is_null($payment_additional_information) && isset($payment_additional_information[self::METHOD_CODE])
            && isset($payment_additional_information[self::AUTHORIZED_PAYMENT_METHOD]))
        {
            $klarnaMethod = $payment_additional_information[self::METHOD_CODE].'_'.$payment_additional_information[self::AUTHORIZED_PAYMENT_METHOD];
            if(isset(self::MAPPED_KLARNA_PAYMENT[$klarnaMethod]))
            {
                $affectedRows = $this->connection->update(
                    $orderPaymentTable,
                    ['cc_type' => self::MAPPED_KLARNA_PAYMENT[$klarnaMethod]],
                    ['parent_id = ?' => $order->entity_id]);

                if($affectedRows > 0)
                {
                    $affectedRows += $this->connection->update(
                        $orderTableGrid,
                        ['payment_cctype' => self::MAPPED_KLARNA_PAYMENT[$klarnaMethod]],
                        ['increment_id = ?' => $order->increment_id]);
                }
            }
        }

        return $affectedRows > 0;
    }

    public function loadOrders()
    {
        $orderTable = $this->connection->getTableName('sales_order');
        $orderPaymentTable = $this->connection->getTableName('sales_order_payment');

        $query = "SELECT so.entity_id, so.increment_id, sop.method, sop.additional_information
FROM {$orderTable} so
INNER JOIN {$orderPaymentTable} sop ON so.entity_id = sop.parent_id
WHERE
      sop.method IN ('klarna_kp') AND
      (sop.cc_type IS NULL OR sop.cc_type NOT LIKE '%klarna%')";

        $ordersToBeRectified = $this->connection->fetchAll($query, [], \Zend_Db::FETCH_OBJ);
        return $ordersToBeRectified;
    }
}
